package com.javamonkey.test;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.lang.reflect.*;
import com.javamonkey.utils.Debug;

/**
 * A simple implementation of Test that runs a single method that
 * returns a boolean. This class is pretty tightly coupled with
 * AutoTest.
 *
 * @see com.javamonkey.test.AutoTest
 *
 * @version $Id$
 */
public class SingleMethodTest /*extends BaseTest implements Test*/ {

  private final static boolean debug = true;

  /**
   * The method.
   */
  private Method method;

  /**
   * The object on which we will invoke the test method.
   */
  private AutoTest autoTest;
  
  /**
   * The array of arguments to be passed to the method.
   */
  private Object[] arguments;

  private Boolean result;
  
  /**
   * Make a new SingleMethodTest that will run the given method on the
   * given object. The method must take no arguments.
   *
   * @param m The method.
   * @param t The autoTest.
   */
  public SingleMethodTest(Method m, AutoTest t) {
    this(m, t, null);
  }

  /**
   * Make a new SingleMethodTest that will run the given method on the
   * given autoTest with the given arguments.
   *
   * @param m The method.
   * @param t The autoTest.
   * @param args The arguments.
   */
  public SingleMethodTest(Method m, AutoTest t, Object[] args) {
    //super(m.getDeclaringClass().getName(), 1);

    if (debug) {
      int numOfArgs = m.getParameterTypes().length;
      if (numOfArgs == 0) {
	Debug.assert(args == null);
      } else {
	Debug.assert(numOfArgs == args.length);
      }
    }
    
    method    = m;
    autoTest  = t;
    arguments = args;
  }
  
  String getMethodName() { return method.getName(); }
  
  public void run() {
    String name = method.getName();
    try {
      result = (Boolean)method.invoke(autoTest, arguments);

      if (result.booleanValue()) {
	autoTest.reportSuccess(name, autoTest.getLastMessage());
      } else {
	autoTest.reportFailure(name, autoTest.getLastMessage(), autoTest.getLastException());
      }
    } catch (InvocationTargetException ite) {
      Throwable t = ite.getTargetException();
      if (t instanceof TestAbortedException) {
	autoTest.reportAbort(name, t);
      } else if (t instanceof PreconditionNotMetException) {
	autoTest.reportSkip(name);
      } else {
	autoTest.reportFailure(name, "Uncaught exception from test method", t);
      }
    } catch (IllegalArgumentException iae) {
      autoTest.reportSkip(name, iae);
    } catch (IllegalAccessException iae) {
      autoTest.reportSkip(name, iae);
    } catch (PreconditionNotMetException pnme) {
      autoTest.reportSkip(name, pnme);
    }

    if (result == null) {
      result = Boolean.FALSE; // We didn't pass, so 
    }
  }

  /**
   * Has this test run yet.
   */
  boolean hasRun() {
    return result != null;
  }

//    void markAsFailed() {
//      result = Boolean.FALSE;
//    }

  /**
   * Did the test pass?
   */
  boolean passed() throws NotRunYetException {
    if (result == null) {
      throw new NotRunYetException();
    }
    return result.booleanValue();
  }

}








