package com.javamonkey.utils.iterators;

/*
 * Copyright (c) 1999 Peter Seibel
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

import java.util.*;
import com.javamonkey.utils.*;
import com.javamonkey.utils.collections.Stack;

/**
 * A base class for iterators that iterate over a naturally recursive
 * data structure such as trees.
 *
 * @version $Id$
 */
public abstract class RecursiveIterator extends PeekingIterator
  implements Iterator
{

  private final static boolean debug = true;
  
  public final static int DEPTH_FIRST = 0;
  public final static int BREADTH_FIRST = 1;

  private boolean depthFirst;
  private boolean includeInteriorNodes = true;

  private Iterator current = null;

  /**
   * A stack of iterators. When the last iterator is empty, then we
   * return END.
   */
  private Stack iterators = new Stack();

  /**
   * Make a new RecursiveIterator that iterates depth first.
   */
  public RecursiveIterator() {
    this(DEPTH_FIRST);
  }
  
  /**
   * Make a new RecursiveIterator that iterates in either depth-first
   * or breadth-first mode.
   *
   * @param type One of DEPTH_FIRST or BREADTH_FIRST to indicate which
   * order the iterator should traverse the tree.
   */
  public RecursiveIterator(int type) {
    if (debug) assertLegalType(type);
    depthFirst = type == DEPTH_FIRST;
  }

  protected Object nextObject() {

    if (done()) return END;

    Iterator current = getCurrentIterator();
    Object o = current.next();

    if (hasChildren(o)) {

      if (depthFirst)  {
	iterators.push(current);
	iterators.push(childrenIterator(o));
      } else {
	iterators.push(childrenIterator(o));
	iterators.push(current);
      }
      if (!includeInteriorNodes) {
	return nextObject();
      }
    }

    return o;
  }

  protected abstract boolean hasChildren(Object o);
  protected abstract Iterator childrenIterator(Object o);

  private boolean done() {
    return getCurrentIterator() == null && iterators.isEmpty();
  }
    
  private Iterator getCurrentIterator() {
    while (!iterators.isEmpty() && !((Iterator)iterators.peek()).hasNext()) {
      iterators.pop();
    }
    return iterators.isEmpty() ? null : (Iterator)iterators.peek();
  }


  // =================================================================
  // Debugging methods.
  
  private final static void assertLegalType(int t) {
    if (t < DEPTH_FIRST || t > BREADTH_FIRST) {
      throw new AssertionError(t + " not a legal type");
    }
  }
}







